from digital_life.models import MemoryCardGenerate,MemoryCardGenerateEasy, MemoryCard2, MemoryCard, MemoryCardScore, MemoryCards, Document, Chapter, MemoryCardGenerate2




def test_1():
    a = {
    "title": "关于高中时光家人支持的回忆",
    "chapters": [
    ]
    }
    Document(**a)
